/************************************************************************\
* Name:  sockserv.c                                                      *
* Description:  This program is a sample program that shows how to use   *
*               the sockets APIs to implement a sockets server.          *
\************************************************************************/

#define SOCKET int
#define FAILURE -1

#include <netinet/in.h>
#include <netdb.h>
#include <netinet/tcp.h>
#include <sys/socket.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

void main(void)
{
   SOCKET  ServerSocket;
   SOCKET  ClientSocket;
   struct  sockaddr_in      ServerInetAddress;
   int     InetAddressLen = sizeof(ServerInetAddress);

   #define BufferSize 1024
   char    Buffer[BufferSize];
   int     BytesRead;
   char    ResponseMessage[17] = "Message received";
   int     WellKnownPort = 5003;
   struct  linger linger;
   int     nodelay = 1;

   /*****************************************\
   * Allocate the server's socket descriptor *
   \*****************************************/
    if ((ServerSocket = socket(AF_INET,SOCK_STREAM, 0)) == FAILURE) {
        perror("socket");
        exit(FAILURE);
    }

   /**************************************************\
   * Set the linger option to force immediate closing *
   \**************************************************/
   linger.l_onoff = 1;  /* turn on timer     */
   linger.l_linger = 0; /* close immediately */
   if ((setsockopt(ServerSocket, SOL_SOCKET, SO_LINGER,
                  (char *)(&linger), sizeof(linger))) == FAILURE) {
        perror("setsockopt");
        exit(FAILURE);
    }

   /**************************************************\
   * Don't buffer up requests - send them immediately *
   \**************************************************/
    if ((setsockopt(ServerSocket, IPPROTO_TCP, TCP_NODELAY,
                   (char *)(&nodelay), sizeof(nodelay))) == FAILURE) {
        perror("setsockopt");
        exit(FAILURE);
    }

   /**************************************************\
   * Initialize the socket address so we can bind it  *
   \**************************************************/
    memset(&ServerInetAddress, 0x00, sizeof(ServerInetAddress));
    ServerInetAddress.sin_family = AF_INET;
    ServerInetAddress.sin_addr.s_addr = INADDR_ANY; /* Let anyone connect  */
    ServerInetAddress.sin_port = htons(WellKnownPort); /* put in net order */

   /********************************************\
   * Bind socket to the well-known port number  *
   \********************************************/
    if (bind(ServerSocket, (struct sockaddr*)&ServerInetAddress,
             sizeof(ServerInetAddress)) == FAILURE) {
        perror("bind");
        exit(FAILURE);
    }

   /*******************************\
   * Show we are willing to listen *
   \*******************************/
    if (listen(ServerSocket, 1) == FAILURE) {
        perror("listen");
        exit(FAILURE);
    }

   /************************************\
   * Accept incoming client connections *
   \************************************/
    if ((ClientSocket = accept(ServerSocket, (struct sockaddr*)
                          &ServerInetAddress, &InetAddressLen)) == FAILURE) {
        perror("accept");
        exit(FAILURE);
    }

   /******************************\
   * Receive data from the client *
   \******************************/
    if ((BytesRead = recv(ClientSocket, Buffer, BufferSize, 0)) == FAILURE) {
        perror("recv");
        exit(FAILURE);
   }
   else { /* We received data */
        Buffer[BytesRead] = '\0'; /* Null terminate the string for printing */
        printf("Message received: %s \n", Buffer);
   }

   /****************\
   * Send response  *
   \****************/
   strcpy(Buffer, ResponseMessage);
    if (send(ClientSocket, Buffer, strlen(ResponseMessage), 0) == FAILURE) {
        perror("send");
        exit(FAILURE);
   }

   /*****************************\
   * Close the client connection *
   \*****************************/
    close(ClientSocket);
   /*************************************************\
   * Close the server connection - only when no more *
   * connection requests will be accepted            *
   \*************************************************/
    close(ServerSocket);

}
