      *_> CNLLSTSPLF SRCFILE(@2/@1) SRCMBR(@3)
      *_> DLTMOD MODULE(@5/@4)
      *_> CRTRPGMOD MODULE(@5/@4) SRCFILE(@2/@1) SRCMBR(@3) +
      *_>   DBGVIEW(@9) OPTIMIZE(@8) OPTION(*EVENTF)
      * ---------------------------------------------------------------
      * Module.: XVSCKC_M01           Project.....:
      * Author.: A. Campin            Date written: 06/13/2008
      * Purpose: This module provides a wrapper around a "C"
      *            implementation of a socket client to make it easy
      *            to interface to RPG programs by providing varying
      *            string, value parms, optional parms and elimenating
      *            parms that do not need to included.
      * ---------------------------------------------------------------
      * Called by: Anyone requiring service
      * ---------------------------------------------------------------
      * Revision history:
      *  Project # Pgmr        Date   Desc
      *
      * End Revision History
      * ---------------------------------------------------------------
     h nomain

      /copy *libl/qsrcf,cb_StdType
      /copy *libl/qsrcf,cb_Std_Con
      /copy *libl/qsrcf,xverrh_pr
      /copy *libl/qsrcf,xvsckc_pr

     d MaximumBufferLength...
     d                 s                   Like(StdInt)
     d                                     Inz(0)
     d ConnectionOpen...
     d                 s                   Like(StdLgl)
     d                                     Inz(cFalse)
      * ---------------------------------------------------------------
      * SCKC_Open
      *   Open connection to TCP socket port.
      *  In     Parms - System Name or TCP I/P Address.
      *                 Port Number to open in.
      *                 Length of buffer used for communications.
      *                 Use Network Byte Order.
      *                 Local CCSID number (Optional).
      *                 Remote CCSID number (Optional).
      *  In/Out Parms - None.
      *  Out    Parms - None.
      *  Return       - None.
      * ---------------------------------------------------------------
     p SCKC_Open...
     p                 b                   Export
     d                 pi
     d   InSystemName...
     d                              256    Varying
     d                                     Value
     d   InPortNumber...
     d                                     Like(StdInt)
     d                                     Value
     d   InMaximumBufferLength...
     d                                     Like(StdInt)
     d                                     Value
     d   InUseNetworkOrder...
     d                                     Like(StdLgl)
     d                                     Value
     d   InLocalCCSID...
     d                                     Like(StdInt)
     d                                     Options(*NoPass)
     d                                     Value
     d   InRemoteCCSID...
     d                                     Like(StdInt)
     d                                     Options(*NoPass)
     d                                     Value

     d Open_TCP_Connection...
     d                 pr                  ExtProc('TCPOPEN')
     d   PR_SystemName...
     d                              256    Options(*VarSize)
     d                                     Const
     d   PR_SystemNameLength...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_PortNumber...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_BufferLength...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_NetworkOrder...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_LocalCCSID...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_RemoteCCSID...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_ReturnCode...
     d                                     Like(StdInt)

     d UXV0029Data...
     d                 ds                  Qualified
     d   ReasonCode...
     d                                     Like(StdInt)
     d   LocalCCSID...
     d                                     Like(StdInt)
     d   RemoteCCSID...
     d                                     Like(StdInt)
     d   MessageId...
     d                                     Like(StdMsgId)

     d NetworkOrder...
     d                 s                   Like(StdInt)
     d LocalCCSID...
     d                 s                   Like(InLocalCCSID)
     d                                     Inz(0)
     d RemoteCCSID...
     d                 s                   Like(InRemoteCCSID)
     d                                     Inz(0)
     d ReturnCode...
     d                 s                   Like(StdInt)
      /Free

       // Must be given port number.
       If InPortNumber <= 0;
         UXV0029Data.ReasonCode = 1;
         ERRH_Throw('UXV0029'         :
                    cXVMessageFileName:
                    UXV0029Data       );
       EndIf;

       // Must be given maximum buffer length.
       If InMaximumBufferLength <= 0;
         UXV0029Data.ReasonCode = 2;
         ERRH_Throw('UXV0029'         :
                    cXVMessageFileName:
                    UXV0029Data       );
       EndIf;

       // Must be either no CCSID's passed or both passed.
       If Not (%Parms = 4 or %Parms = 6);
         UXV0029Data.ReasonCode = 3;
         ERRH_Throw('UXV0029'         :
                    cXVMessageFileName:
                    UXV0029Data       );
       EndIf;

       // Remote and local CCSID can be zero but if given, both must be supplied.
       If %Parms = 6;
         LocalCCSID  = InLocalCCSID;
         RemoteCCSID = InRemoteCCSID;
         If (RemoteCCSID <> 0 and
             LocalCCSID = 0) or
            (LocalCCSID <> 0 and
             RemoteCCSID = 0);
           UXV0029Data.ReasonCode  = 4;
           UXV0029Data.LocalCCSID  = InLocalCCSID;
           UXV0029Data.RemoteCCSID = InRemoteCCSID;
           ERRH_Throw('UXV0029'         :
                      cXVMessageFileName:
                      UXV0029Data       );
         EndIf;
       EndIf;

       If InUseNetworkOrder;
         NetworkOrder = 1;
       Else;
         NetworkOrder = 0;
       EndIf;

       Open_TCP_Connection(InSystemName         :
                           %Len(InSystemName)   :
                           InPortNumber         :
                           InMaximumBufferLength:
                           NetworkOrder         :
                           LocalCCSID           :
                           RemoteCCSID          :
                           ReturnCode           );
       If ReturnCode <> 0;
         UXV0029Data.ReasonCode = 5;
         UXV0029Data.MessageId  = 'CPE' + %Trim(%EditC(ReturnCode:'Z'));
         ERRH_Throw('UXV0029'         :
                    cXVMessageFileName:
                    UXV0029Data       );
       EndIf;

       MaximumBufferLength   = InMaximumBufferLength;
       ConnectionOpen = cTrue;

       Return;

      /End-Free
     p                 e
      * ---------------------------------------------------------------
      * SCKC_Close
      *   Close connection to TCP socket port.
      *  In     Parms - None.
      *  In/Out Parms - None.
      *  Out    Parms - None.
      *  Return       - None.
      * ---------------------------------------------------------------
     p SCKC_Close...
     p                 b                   Export
     d                 pi

     d Close_TCP_Connection...
     d                 pr                  ExtProc('TCPCLOSE')
     d   PR_ReturnCode...
     d                                     Like(StdInt)

     d UXV0030Data...
     d                 ds                  Qualified
     d   ReasonCode...
     d                                     Like(StdInt)
     d   MessageId...
     d                                     Like(StdMsgId)

     d ReturnCode...
     d                 s                   Like(StdInt)
      /Free

       If ConnectionOpen;
         Close_TCP_Connection(ReturnCode);
         If ReturnCode <> 0;
           UXV0030Data.ReasonCode = 1;
           UXV0030Data.MessageId  = 'CPE' + %Trim(%EditC(ReturnCode:'Z'));
           ERRH_Throw('UXV0030'         :
                      cXVMessageFileName:
                      UXV0030Data       );
         EndIf;
       EndIf;

       MaximumBufferLength   = 0;
       ConnectionOpen = cFalse;

       Return;

      /End-Free
     p                 e
      * ---------------------------------------------------------------
      * SCKC_Send
      *   Send data to a TCP/IP socket.
      *  In     Parms - Send Buffer
      *                 Send Option.
      *                    cSCKC_SendUsingTwoByteHeader  - Include two byte header with length.
      *                    cSCKC_SendUsingFourByteHeader - Include four byte header with length.
      *                    cSCKC_SendUsingBufferLength   - Send full length of buffer.
      *  In/Out Parms - None.
      *  Out    Parms - None.
      *  Return       - None.
      * ---------------------------------------------------------------
     p SCKC_Send...
     p                 b                   Export
     d                 pi
     d   InSendBuffer...
     d                            32767    Varying
     d                                     Const
     d   InSendOption...
     d                                     Like(StdChr)
     d                                     Value
     d                                     Options(*NoPass)

     d Send_TCP_Data...
     d                 pr                  ExtProc('TCPSEND')
     d   PR_SendBuffer...
     d                            32767    Options(*VarSize)
     d                                     Const
     d   PR_SendLength...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_SendOption...
     d                                     Like(StdChr)
     d                                     Const
     d   PR_ReturnCode...
     d                                     Like(StdInt)

     d UXV0031Data...
     d                 ds                  Qualified
     d   ReasonCode...
     d                                     Like(StdInt)
     d   MessageId...
     d                                     Like(StdMsgId)
     d   BufferLength...
     d                                     Like(StdInt)
     d   SendBufferLength...
     d                                     Like(StdInt)
     d   SendOption...
     d                                     Like(StdChr)

     d SendOption...
     d                 s                   Like(StdChr)
     d ReturnCode...
     d                 s                   Like(StdInt)
      /Free

       // Connection must be open.
       If Not ConnectionOpen;
         UXV0031Data.ReasonCode  = 1;
         ERRH_Throw('UXV0031'         :
                    cXVMessageFileName:
                    UXV0031Data       );
       EndIf;

       // Buffer must be less than or equal to size of buffer specified
       //  when connection opened.
       If %Len(InSendBuffer) > MaximumBufferLength;
         UXV0031Data.ReasonCode       = 2;
         UXV0031Data.BufferLength     = MaximumBufferLength;
         UXV0031Data.SendBufferLength = %Len(InSendBuffer);
         ERRH_Throw('UXV0031'         :
                    cXVMessageFileName:
                    UXV0031Data       );
       EndIf;

       If %Parms = 3;
         // Send option must one of defined values.
         If Not((InSendOption = cSCKC_SendUsingTwoByteHeader) or
                (InSendOption = cSCKC_SendUsingFourByteHeader) or
                (InSendOption = cSCKC_SendUsingBufferLength));
           UXV0031Data.ReasonCode = 3;
           UXV0031Data.SendOption = InSendOption;
           ERRH_Throw('UXV0031'         :
                      cXVMessageFileName:
                      UXV0031Data       );
         EndIf;

         If InSendOption = cSCKC_SendUsingBufferLength;
           SendOption = *Blank;
         Else;
           SendOption = InSendOption;
         EndIf;
       EndIf;

       Send_TCP_Data(InSendBuffer      :
                     %Len(InSendBuffer):
                     SendOption        :
                     ReturnCode        );
       If ReturnCode <> 0;
         UXV0031Data.ReasonCode = 4;
         UXV0031Data.MessageId  = 'CPE' + %Trim(%EditC(ReturnCode:'Z'));
         ERRH_Throw('UXV0031'         :
                    cXVMessageFileName:
                    UXV0031Data       );
       EndIf;

       Return;

      /End-Free
     p                 e
      * ---------------------------------------------------------------
      * SCKC_Receive
      *   Receive data from a TCP/IP socket.
      *  In     Parms - Receive buffer size. Length of buffer to pass to socket server.
      *  In     Parms - Receive Option.
      *                     cSCKC_ReceiveUsingTwoByteHeader  - Receive using two byte length in head
      *                     cSCKC_ReceiveUsingFourByteHeader - Receive using four byte length in hea
      *                     cSCKC_ReceiveUsingSimpleReceive  - Do a simple receive for length of buf
      *                     cSCKC_ReceiveUsingFillBuffer     - Fill buffer using multiple receives i
      *                     cSCKC_ReceiveUsingDelimitator    - Receive until delimitator seen or buf
      *                 Delimitator (Optional-Must be present if D option).
      *  In/Out Parms - None.
      *  Out    Parms - Receive Buffer. Returns with actual length of string.
      *  Return       - None.
      * ---------------------------------------------------------------
     p SCKC_Receive...
     p                 b                   Export
     d                 pi
     d   OutReceiveBuffer...
     d                            32767    Varying
     d   InBufferLengthToPass...
     d                                     Like(StdInt)
     d                                     Value
     d   InReceiveOption...
     d                                     Like(StdChr)
     d                                     Value
     d   InDelimitator...
     d                                     Like(StdChr)
     d                                     Options(*NoPass)
     d                                     Value

     d Receive_TCP_Data...
     d                 pr                  ExtProc('TCPRECV')
     d   PR_ReceiveBuffer...
     d                            32767    Options(*VarSize)
     d   PR_ReceiveLength...
     d                                     Like(StdInt)
     d                                     Const
     d   PR_ReceiveOption...
     d                                     Like(StdChr)
     d                                     Const
     d   PR_Delimitator...
     d                                     Like(StdChr)
     d                                     Const
     d   PR_ReturnLength...
     d                                     Like(StdInt)
     d   PR_ReturnCode...
     d                                     Like(StdInt)

     d UXV0032Data...
     d                 ds                  Qualified
     d   ReasonCode...
     d                                     Like(StdInt)
     d   MessageId...
     d                                     Like(StdMsgId)
     d   BufferSize...
     d                                     Like(StdInt)
     d   ReceiveBufferSize...
     d                                     Like(StdInt)
     d   ReceiveOption...
     d                                     Like(StdChr)

     d ReceiveBuffer...
     d                 s          32767    Based(ptrReceiveBuffer)
     d Delimitator...
     d                 s                   Like(StdChr)
     d ReturnLength...
     d                 s                   Like(StdInt)
     d ReturnCode...
     d                 s                   Like(StdInt)
      /Free

       // Connection must be open.
       If Not ConnectionOpen;
         UXV0032Data.ReasonCode  = 1;
         ERRH_Throw('UXV0032'         :
                    cXVMessageFileName:
                    UXV0032Data       );
       EndIf;

       // Send option must one of defined values.
       If Not((InReceiveOption = cSCKC_ReceiveUsingTwoByteHeader)  or
              (InReceiveOption = cSCKC_ReceiveUsingFourByteHeader) or
              (InReceiveOption = cSCKC_ReceiveUsingSimpleReceive)  or
              (InReceiveOption = cSCKC_ReceiveUsingFillBuffer)     or
              (InReceiveOption = cSCKC_ReceiveUsingDelimitator));
         UXV0032Data.ReasonCode    = 2;
         UXV0032Data.ReceiveOption = InReceiveOption;
         ERRH_Throw('UXV0032'         :
                    cXVMessageFileName:
                    UXV0032Data       );
       EndIf;

       // If fill using delimitator, delimitator must be passed.
       If InReceiveOption = cSCKC_ReceiveUsingDelimitator and
         %Parms = 3;
           UXV0032Data.ReasonCode  = 3;
           ERRH_Throw('UXV0032'         :
                      cXVMessageFileName:
                      UXV0032Data       );
       EndIf;

       // If delimitator passed, load passed into working, else just blank.
       If %Parms = 4;
         Delimitator = InDelimitator;
       Else;
         Delimitator = *Blanks;
       EndIf;

       // Buffer length to pass must be greater than zero and not longer than the buffer
       //  length specified on SCKC_Open.
       If InBufferLengthToPass <= 0 or
          InBufferLengthToPass > MaximumBufferLength;
         UXV0032Data.ReasonCode        = 4;
         UXV0032Data.MessageId         = *Blanks;
         UXV0032Data.BufferSize        = MaximumBufferLength;
         UXV0032Data.ReceiveBufferSize = InBufferLengthToPass;
         ERRH_Throw('UXV0032'         :
                    cXVMessageFileName:
                    UXV0032Data       );
       EndIf;

       // Define work buffer to pass.
       ptrReceiveBuffer = %Alloc(InBufferLengthToPass);

       Receive_TCP_Data(ReceiveBuffer       :
                        InBufferLengthToPass:
                        InReceiveOption     :
                        Delimitator         :
                        ReturnLength        :
                        ReturnCode          );
       If ReturnCode <> 0;
         DeAlloc ptrReceiveBuffer;
         UXV0032Data.ReasonCode = 5;
         UXV0032Data.MessageId  = 'CPE' + %Trim(%EditC(ReturnCode:'Z'));
         ERRH_Throw('UXV0032'         :
                    cXVMessageFileName:
                    UXV0032Data       );
       EndIf;

       OutReceiveBuffer = %Subst(ReceiveBuffer:
                                 1            :
                                 ReturnLength );
       DeAlloc ptrReceiveBuffer;

       Return;

      /End-Free
     p                 e
