/*    *_> CNLLSTSPLF SRCFILE(@2/@1) SRCMBR(@3)                       */
/*    *_> DLTMOD MODULE(@5/@4)                                       */
/*    *_> CRTCMOD MODULE(@5/@4) SRCFILE(@2/@1) SRCMBR(@3) +          */
/*    *_>   DBGVIEW(*LIST) OPTION(*EVENTF)                           */

     #include <stdio.h>
     #include <stdlib.h>
     #include <string.h>
     #include <sys/types.h>
     #include <sys/socket.h>
     #include <netinet/in.h>
     #include <netinet/tcp.h>
     #include <arpa/inet.h>
     #include <netdb.h>
     #include <errno.h>
     #include <iconv.h>
     #include <qtqiconv.h>

     /* Global variables for socket descriptors and translation */
     /* information                                             */

     int l_ccsid = 0;               /* Local System CCSID  */
     int r_ccsid = 0;               /* Remote System CCSID */
     iconv_t e_a_cd;                /* EBCDIC to ASCII conversion */
     iconv_t a_e_cd;                /* ASCII to EBCDIC conversion */
     char *ascii_buffer = NULL;     /* Pointer to buffer for conversion */
     int buffer_length = 0;         /* Current buffer size */
     int tcp_socket;                /* Current active socket descriptor */
     int translate;                 /* Translation flag */

     /* Initialize CCSID structure for code page conversion APIs */

     void fill_QtqCode(QtqCode_T *ccode, int ccsid)
     {
       memset(ccode, 0, sizeof(QtqCode_T));
       ccode->CCSID = ccsid;
     }

     /****************************************/
     /* Initialize code page conversion APIs */
     /****************************************/
     void set_translation(int local_ccsid, int remote_ccsid)
     {
       QtqCode_T local;
       QtqCode_T remote;

       l_ccsid = local_ccsid;             /* Keep global information */
       r_ccsid = remote_ccsid;

       fill_QtqCode(&local, local_ccsid);    /* Fill API structures */
       fill_QtqCode(&remote, remote_ccsid);

       e_a_cd = QtqIconvOpen(&remote, &local);  /* Initialize conversion */
       a_e_cd = QtqIconvOpen(&local, &remote);
     }

     /**********************************************/
     /* Open connection to listening remote system */
     /**********************************************/
     int open_connection(char *name,
                         int length,
                         int iport,
                         int network_order,
                         int *error)
     {
       char hname[257];
       struct hostent *host;
       struct sockaddr_in addr;

       *error = 0;

       memcpy(hname, name, length);   /* Make sure name has ending */
       hname[length] = '\0';          /* delimiter */

       /* Get socket descriptor */

       if ((tcp_socket = socket(AF_INET, SOCK_STREAM, 0)) == -1)
       {
         *error = errno;
         return -1;
       }

       /* Set up socket to connect to remote system */

       addr.sin_family = AF_INET;
       if (network_order)
         addr.sin_port = htons((unsigned short)iport);
       else
         addr.sin_port = (unsigned short)iport;

       memset(addr.sin_zero, 0x00, sizeof(addr.sin_zero));

       if ((host = gethostbyname(hname)) == NULL)  /* Check by name */
       {                                         /* otherwise by address */
         if ((addr.sin_addr.s_addr = inet_addr(hname)) == 0xFFFFFFFF)
         {
           *error = errno;
           return -1;
         }
       }
       else
         memcpy(&addr.sin_addr, host->h_addr, host->h_length);

       /* Connect to remote system */

       if (connect(tcp_socket, (struct sockaddr *)&addr, sizeof(addr)   ) != 0)
       {
         *error = errno;
         return -1;
       }

       return 0;
     }

     /********************************************************************/
     /* Receive a specified number of bytes, concatenating from multiple */
     /* TCP/IP receives if necessary                                     */
     /********************************************************************/
     void fill_receive(char *buffer, int length, int *rc)
     {
       int rlength;

       *rc = 0;

       while (length > 0 && *rc == 0)
       {
         rlength = recv(tcp_socket, buffer, length, 0);
         if (rlength > 0)
         {
           length -= rlength;
           buffer += rlength;
         }
         else if (rlength == 0)
           *rc = ENOTCONN;
         else
           *rc = errno;
       }
     }

     /**************************************************************/
     /* Receive until delimiting character found or buffer is full */
     /**************************************************************/
     int delimit_receive(char *buffer, int length, char delim, int *rc)
     {
       int rlength;
       char *rdelim;
       int tlength = 0;

       *rc = 0;

       while (length > 0 && *rc == 0)
       {
         rlength = recv(tcp_socket, buffer, length, 0);
         if (rlength > 0)
         {
           if ((rdelim = memchr(buffer, delim, rlength)) != NULL)
           {
             tlength += (rdelim - buffer) + 1;
             break;
           }
           length -= rlength;
           buffer += rlength;
           tlength += rlength;
         }
         else if (rlength == 0)
           *rc = ENOTCONN;
         else
           *rc = errno;
       }

       return tlength;
     }

     /**************************************************/
     /* TCPOPEN - Open a connection to a remote system */
     /**************************************************/
     void TCPOPEN(char *iname,
                  int  *ilength,
                  int  *iport,
                  int  *blength,
                  int  *network_order,
                  int  *local_ccsid,
                  int  *remote_ccsid,
                  int  *rc)
     {
       int error;

       /* Set up translation if required */

       if (*remote_ccsid)
       {
         if ((l_ccsid != *local_ccsid) || (r_ccsid != *remote_ccsid))
           set_translation(*local_ccsid, *remote_ccsid);
         translate = 1;
       }
       else
         translate = 0;

       if (*blength > buffer_length)  /* Grow buffer if needed */
       {
         buffer_length = *blength;
         ascii_buffer = realloc(ascii_buffer, buffer_length);
       }

       /* Open the connection */

       if (open_connection(iname,
                           *ilength,
                           *iport,
                           *network_order,
                           &error) == -1)
       {
         *rc = error;
         return;
       }
       else
         *rc = 0;
     }

     /****************************************/
     /* TCPSEND - Send data to remote system */
     /****************************************/
     void TCPSEND(char *buffer,
                  int  *length,
                  char *opt,
                  int  *rc)
     {
       *rc = 0;

       if (*opt == '2')   /* Send 2 byte length field first */
       {
         short slength = (short)(*length);
         if (send(tcp_socket, (char *)&slength, sizeof(slength), 0) == -1)
         {
           *rc = errno;
           return;
         }
       }
       else if (*opt == '4')  /* Send 4 byte length field first */
       {
         if (send(tcp_socket, (char *)length, sizeof(int), 0) == -1)
         {
           *rc = errno;
           return;
         }
       }

       /* Translate data if necessary and send */

       if (translate)
       {
         char *a_buffer = ascii_buffer;
         char *e_buffer = buffer;
         unsigned isize = *length;
         unsigned osize = *length;

         if (*length > buffer_length)
         {
           *rc = EINVAL;
           *length = 0;
           return;
         }

         iconv(e_a_cd, &e_buffer, &isize, &a_buffer, &osize);

         if (send(tcp_socket, ascii_buffer, *length, 0) == -1)
         {
           *rc = errno;
           return;
         }
       }
       else
       {
         if (send(tcp_socket, buffer, *length, 0) == -1)
         {
           *rc = errno;
           return;
         }
       }
     }

     /**************************************************/
     /* TCPRECV - Receive data using specified options */
     /**************************************************/
     void TCPRECV(char *buffer,
                  int  *size,
                  char *opt,
                  char *delim,
                  int  *length,
                  int  *rc)
     {
       char *recv_buff;
       short slength;

       *rc = 0;

       /* Check for translation space */

       if (translate)
       {
         if (*size > buffer_length)
         {
           *rc = EINVAL;
           *length = 0;
           return;
         }
         recv_buff = ascii_buffer;
       }
       else
         recv_buff = buffer;

       switch (*opt)
       {
         /* Data will be preceded by two byte length */
         case '2' : fill_receive((char *)&slength, sizeof(slength), rc);
                    if (*rc == 0)
                    {
                      if (slength <= *size)
                        *length = slength;
                      else
                        *length = *size;
                      fill_receive(recv_buff, *length, rc);
                    }
                    break;
         /* Data will be preceded by four byte length */
         case '4' : fill_receive((char *)length, sizeof(int), rc);
                    if (*rc == 0)
                    {
                      if (*length > *size)
                        *length = *size;
                      fill_receive(recv_buff, *length, rc);
                    }
                    break;
         /* Just do a single receive */
         case 'S' : *length = recv(tcp_socket, recv_buff, *size, 0);
                    if (*length == -1)
                      *rc = errno;
                    else if (*length == 0)
                      *rc = ENOTCONN;
                    break;
         /* Receive until buffer is full */
         case 'F' : fill_receive(recv_buff, *size, rc);
                    *length = *size;
                    break;
         /* Receive until delimiter is found */
         case 'D' : *length = delimit_receive(recv_buff, *size,
                                              *delim, rc);
                    break;
         default:   *rc = EINVAL;
                    *length = 0;
                    return;
       }

       /* Translate data if required */

       if (translate)
       {
         char *a_buffer = ascii_buffer;
         char *e_buffer = buffer;
         unsigned isize = *length;
         unsigned osize = *length;

         iconv(a_e_cd, &a_buffer, &isize, &e_buffer, &osize);
       }
     }

     /**************************************/
     /* TCPCLOSE - Close active connection */
     /**************************************/
     void TCPCLOSE(int *rc)
     {
       if (close(tcp_socket))
         *rc = errno;
       else
         *rc = 0;
     }
