      *----------------------------------------------------------------
      * Prototype: XVDATF_PR          Project.....:
      * Author...: A. Campin          Date written: 05/15/2008
      * Purpose..: Prototypes for Common Date Functions.
      *----------------------------------------------------------------
      * Revision history:
      *  Project # Pgmr        Date   Desc
      *
      * End Revision History
      *----------------------------------------------------------------
      * SCKC_Open
      *   Open connection to TCP socket port.
      *  In     Parms - System Name or TCP I/P Address.
      *                 Port Number to open in.
      *                 Length of buffer used for communications.
      *                 Use Network Byte Order.
      *                 Local CCSID number (Optional).
      *                 Remote CCSID number (Optional).
      *  In/Out Parms - None.
      *  Out    Parms - None.
      *  Return       - None.
      *----------------------------------------------------------------
     d SCKC_Open...
     d                 pr                  ExtProc('SCKC_Open')
     d   PR_InSystemName...
     d                              256    Varying
     d                                     Value
     d   PR_InPortNumber...
     d                                     Like(StdInt)
     d                                     Value
     d   PR_InMaximumBufferLength...
     d                                     Like(StdInt)
     d                                     Value
     d   PR_InUseNetworkOrder...
     d                                     Like(StdLgl)
     d                                     Value
     d   PR_InLocalCCSID...
     d                                     Like(StdInt)
     d                                     Options(*NoPass)
     d                                     Value
     d   PR_InRemoteCCSID...
     d                                     Like(StdInt)
     d                                     Options(*NoPass)
     d                                     Value
      *----------------------------------------------------------------
      * SCKC_Close
      *   Close connection to TCP socket port.
      *  In     Parms - None.
      *  In/Out Parms - None.
      *  Out    Parms - None.
      *  Return       - None.
      *----------------------------------------------------------------
     d SCKC_Close...
     d                 pr                  ExtProc('SCKC_Close')

      * For SCKC_Send.
     d cSCKC_SendUsingTwoByteHeader...
     d                 c                   '2'
     d cSCKC_SendUsingFourByteHeader...
     d                 c                   '4'
     d cSCKC_SendUsingBufferLength...
     d                 c                   'L'
      *----------------------------------------------------------------
      * SCKC_Send
      *   Send data to a TCP/IP socket.
      *  In     Parms - Send Buffer
      *                 Send Option.
      *                    cSCKC_SendUsingTwoByteHeader  - Include two byte header with length.
      *                    cSCKC_SendUsingFourByteHeader - Include four byte header with length.
      *                    cSCKC_SendUsingBufferLength   - Send full length of buffer.
      *  In/Out Parms - None.
      *  Out    Parms - None.
      *  Return       - None.
      *----------------------------------------------------------------
     d SCKC_Send...
     d                 pr                  ExtProc('SCKC_Send')
     d    PR_InSendBuffer...
     d                            32767    Varying
     d                                     Const
     d    PR_SendOption...
     d                                     Like(StdChr)
     d                                     Value
     d                                     Options(*NoPass)

      * For SCKC_Receive.
     d cSCKC_ReceiveUsingTwoByteHeader...
     d                 c                   '2'
     d cSCKC_ReceiveUsingFourByteHeader...
     d                 c                   '4'
     d cSCKC_ReceiveUsingSimpleReceive...
     d                 c                   'S'
     d cSCKC_ReceiveUsingFillBuffer...
     d                 c                   'F'
     d cSCKC_ReceiveUsingDelimitator...
     d                 c                   'D'
      *----------------------------------------------------------------
      * SCKC_Receive
      *   Receive data from a TCP/IP socket.
      *  In     Parms - Receive buffer size. Length of buffer to pass to socket server.
      *  In     Parms - Receive Option.
      *                     cSCKC_ReceiveUsingTwoByteHeader  - Receive using two byte length in head
      *                     cSCKC_ReceiveUsingFourByteHeader - Receive using four byte length in hea
      *                     cSCKC_ReceiveUsingSimpleReceive  - Do a simple receive for length of buf
      *                     cSCKC_ReceiveUsingFillBuffer     - Fill buffer using multiple receives i
      *                     cSCKC_ReceiveUsingDelimitator    - Receive until delimitator seen or buf
      *                 Delimitator (Optional-Must be present if D option).
      *  In/Out Parms - None.
      *  Out    Parms - Receive Buffer. Returns with actual length of string.
      *  Return       - None.
      *----------------------------------------------------------------
     d SCKC_Receive...
     d                 pr                  ExtProc('SCKC_Receive')
     d   PR_OutReceiveBuffer...
     d                            32767    Varying
     d   PR_InBufferLengthToPass...
     d                                     Like(StdInt)
     d                                     Value
     d   PR_InReceiveOption...
     d                                     Like(StdChr)
     d                                     Value
     d   PR_InDelimitator...
     d                                     Like(StdChr)
     d                                     Options(*NoPass)
     d                                     Value
